"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fs = void 0;
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const util_1 = tslib_1.__importDefault(require("util"));
const path_1 = tslib_1.__importDefault(require("path"));
const utils_1 = require("./utils");
const ENOENT = 'ENOENT'; // not found
const EEXIST = 'EEXIST';
function copy(src, dist, opts) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        opts = Object.assign({ override: true }, opts);
        const srcStat = yield exports.fs.stat(src);
        const isSkiped = opts.skip ? yield opts.skip(src, srcStat) : false;
        if (isSkiped)
            return;
        // auto merge for directory
        if (srcStat.isDirectory()) {
            yield exports.fs.mkdirp(dist);
            let childs = yield exports.fs.readdir(src);
            yield Promise.all(childs.map(child => copy(path_1.default.join(src, child), path_1.default.join(dist, child), opts)));
        }
        else if (srcStat.isFile() || srcStat.isSymbolicLink()) {
            if (yield exports.fs.lexists(dist)) {
                let isDir = yield exports.fs.isDirectory(dist);
                if (isDir) {
                    dist = path_1.default.join(dist, path_1.default.basename(src));
                }
                else if (opts.override) {
                    yield exports.fs.rmrf(dist);
                }
                else {
                    return;
                }
            }
            else {
                let lastChar = dist[dist.length - 1];
                if (lastChar === '/' || lastChar === '\\') {
                    yield exports.fs.mkdirp(dist);
                    dist = path_1.default.join(dist, path_1.default.basename(src));
                }
                else {
                    let dir = path_1.default.dirname(dist);
                    yield exports.fs.mkdirp(dir);
                }
            }
            yield exports.fs.copyFile(src, dist);
        }
    });
}
function watchDir(dir, options, cb) {
    if (utils_1.Is.fn(options)) {
        cb = options;
        options = void 0;
    }
    options = Object.assign({ persistent: true, threshold: 300 }, options);
    cb = cb && utils_1.promiseQueue(cb);
    if (options.threshold) {
        cb = cb && utils_1.debounce(cb, options.threshold);
    }
    if (process.platform === 'linux') {
        // tslint:disable-next-line:no-floating-promises
        exports.fs.iter(dir, (file) => {
            exports.fs.watch(file, {
                recursive: false,
                persistent: options.persistent,
            }, cb);
        });
    }
    else {
        exports.fs.watch(dir, { recursive: true, persistent: options.persistent }, cb);
    }
}
exports.fs = Object.assign(Object.assign({}, fs_1.default), { access: util_1.default.promisify(fs_1.default.access), open: util_1.default.promisify(fs_1.default.open), rename: util_1.default.promisify(fs_1.default.rename), truncate: util_1.default.promisify(fs_1.default.truncate), rmdir: util_1.default.promisify(fs_1.default.rmdir), mkdir: util_1.default.promisify(fs_1.default.mkdir), readdir: util_1.default.promisify(fs_1.default.readdir), readlink: util_1.default.promisify(fs_1.default.readlink), symlink: util_1.default.promisify(fs_1.default.symlink), lstat: util_1.default.promisify(fs_1.default.lstat), stat: util_1.default.promisify(fs_1.default.stat), link: util_1.default.promisify(fs_1.default.link), unlink: util_1.default.promisify(fs_1.default.unlink), chmod: util_1.default.promisify(fs_1.default.chmod), chown: util_1.default.promisify(fs_1.default.chown), utimes: util_1.default.promisify(fs_1.default.utimes), realpath: util_1.default.promisify(fs_1.default.realpath), mkdtemp: util_1.default.promisify(fs_1.default.mkdtemp), writeFile: util_1.default.promisify(fs_1.default.writeFile), appendFile: util_1.default.promisify(fs_1.default.appendFile), readFile: util_1.default.promisify(fs_1.default.readFile), existsSync: fs_1.default.existsSync, createReadString: fs_1.default.createReadStream, createWriteStream: fs_1.default.createWriteStream, constants: fs_1.default.constants, watchDir,
    copy, exists(path) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                yield exports.fs.stat(path);
            }
            catch (error) {
                if (error.code === ENOENT) {
                    return false;
                }
                else {
                    throw error;
                }
            }
            return true;
        });
    },
    /** exists via lstat, if a symbolic link's target file doesn't exists, `fs.exists` will return false, but `fs.lexists` will return true. */
    lexists(path) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                yield exports.fs.lstat(path);
            }
            catch (error) {
                if (error.code === ENOENT) {
                    return false;
                }
                else {
                    throw error;
                }
            }
            return true;
        });
    },
    isFile(path) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                return (yield exports.fs.lstat(path)).isFile();
            }
            catch (error) {
                if (error.code === ENOENT) {
                    return false;
                }
                else {
                    throw error;
                }
            }
        });
    },
    isDirectory(path) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                return (yield exports.fs.lstat(path)).isDirectory();
            }
            catch (error) {
                if (error.code === ENOENT) {
                    return false;
                }
                else {
                    throw error;
                }
            }
        });
    },
    isSymbolicLink(path) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                return (yield exports.fs.lstat(path)).isSymbolicLink();
            }
            catch (error) {
                if (error.code === ENOENT) {
                    return false;
                }
                else {
                    throw error;
                }
            }
        });
    }, copyFile: fs_1.default.copyFile ? util_1.default.promisify(fs_1.default.copyFile) : (src, dist) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        yield exports.fs.stat(src);
        return new Promise((res, rej) => {
            exports.fs
                .createReadStream(src, { highWaterMark: 2 * 1024 * 1024 })
                .pipe(exports.fs.createWriteStream(dist))
                .on('error', rej)
                .on('close', res);
        });
    }), 
    /**
     * Make directory with parents, like `mkdir -p`
     * @param dir
     */
    mkdirp(dir) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let parent = path_1.default.dirname(dir);
            if (!(yield exports.fs.exists(parent))) {
                yield exports.fs.mkdirp(parent);
            }
            return exports.fs.mkdir(dir).catch(error => {
                if (error.code !== EEXIST) {
                    throw error;
                }
            });
        });
    },
    /**
     * Make directory with parents, like `mkdir -p`
     * @param dir
     */
    mkdirpSync(dir) {
        if (dir === '/')
            return;
        let parent = path_1.default.dirname(dir);
        if (!exports.fs.existsSync(parent)) {
            exports.fs.mkdirpSync(parent);
        }
        if (!exports.fs.existsSync(dir)) {
            try {
                exports.fs.mkdirSync(dir);
            }
            catch (error) {
                if (error.code !== EEXIST) {
                    throw error;
                }
            }
        }
    },
    /**
     * Remove file or directory recursively, like `rm -rf`
     * @param path The path to remove
     * @param opts Options
     */
    rmrf(path) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let stat;
            try {
                stat = yield exports.fs.lstat(path);
            }
            catch (error) {
                if (error.code === ENOENT) {
                    return;
                }
                throw error;
            }
            if (stat.isDirectory()) {
                const children = yield exports.fs.readdir(path);
                yield Promise.all(children
                    .map(child => path_1.default.join(path, child))
                    .map(exports.fs.rmrf));
                yield exports.fs.rmdir(path);
            }
            else {
                yield exports.fs.unlink(path);
            }
        });
    },
    outputFile(path, data, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let dir = path_1.default.dirname(path);
            yield exports.fs.mkdirp(dir);
            return exports.fs.writeFile(path, data, options);
        });
    },
    outputFileSync(path, data, options) {
        let dir = path_1.default.dirname(path);
        exports.fs.mkdirpSync(dir);
        return exports.fs.writeFileSync(path, data, options);
    },
    outputJson(path, data, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const [replacer, space] = utils_1.Is.obj(options)
                ? [options.replacer, options.space]
                : [void 0, void 0];
            return exports.fs.outputFile(path, JSON.stringify(data, replacer, space), options);
        });
    },
    outputJsonSync(path, data, options) {
        const [replacer, space] = utils_1.Is.obj(options)
            ? [options.replacer, options.space]
            : [void 0, void 0];
        return exports.fs.outputFileSync(path, JSON.stringify(data, replacer, space), options);
    },
    readJson(path, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let data = yield exports.fs.readFile(path, options);
            if (!utils_1.Is.str(data)) {
                data = data.toString('utf8');
            }
            return JSON.parse(data);
        });
    },
    readJsonSync(path, options) {
        let data = exports.fs.readFileSync(path, options);
        if (!utils_1.Is.str(data)) {
            data = data.toString('utf8');
        }
        return JSON.parse(data);
    },
    iter(dir, 
    /** return true will skip */
    skip) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let children = yield exports.fs.readdir(dir);
            yield Promise.all(children.map((child) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                let path = path_1.default.join(dir, child);
                let stat = yield exports.fs.stat(path);
                let isSkipped = yield skip(path, stat);
                if (isSkipped)
                    return;
                if (stat.isDirectory()) {
                    yield exports.fs.iter(path, skip);
                }
            })));
        });
    } });
//# sourceMappingURL=fs.js.map