"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.logger = exports.Logger = exports.LogLevels = void 0;
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const util_1 = tslib_1.__importDefault(require("util"));
const strip_ansi_1 = tslib_1.__importDefault(require("strip-ansi"));
const utils_1 = require("./utils");
exports.LogLevels = {
    debug: 0,
    info: 1,
    warn: 2,
    error: 3,
};
function makeLogger(level, logger) {
    return (...args) => {
        const { _props: props } = logger;
        let levelNum = exports.LogLevels[level];
        let filterLevelNum = exports.LogLevels[props.level || 'debug'];
        const time = props.logTime
            ? typeof props.logTime === 'function'
                ? props.logTime()
                : utils_1.formatDate(new Date())
            : '';
        let message = (props.format || Logger.defaultProps.format)(level, time, props.levelColor[level], args);
        if (levelNum >= filterLevelNum && !props.hideConsole) {
            console.log(message);
        }
        props.onLog &&
            props.onLog({
                level,
                time: new Date(),
                formatedTime: time,
                message: strip_ansi_1.default(message),
                levelNum,
                filterLevelNum,
                args,
            });
    };
}
class Logger {
    constructor(_props = {}) {
        this.debug = makeLogger('debug', this);
        this.info = makeLogger('info', this);
        this.log = makeLogger('info', this);
        this.warn = makeLogger('warn', this);
        this.error = makeLogger('error', this);
        this._props = Object.assign(Object.assign({}, Logger.defaultProps), _props);
    }
    static get defaultProps() {
        return {
            logTime: false,
            levelColor: {
                debug: chalk_1.default.blueBright,
                info: chalk_1.default.green,
                warn: chalk_1.default.yellow,
                error: chalk_1.default.red,
            },
            format(level, time, color, args) {
                return `${color(`[${level}]`)}${time ? ' ' + time : ''} ${args
                    .map((a) => (typeof a === 'string' ? a : util_1.default.inspect(a, false, 5)))
                    .join(' ')}`;
            },
            level: 'debug',
        };
    }
}
exports.Logger = Logger;
exports.logger = new Logger();
//# sourceMappingURL=logger.js.map